@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2017 Aurimas Cernius
+ * Copyright (C) 2012-2014,2017,2020 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <glibmm/regex.h>
 
 #include "debug.hpp"
+#include "ignote.hpp"
 #include "notedirectorywatcherapplicationaddin.hpp"
 #include "notedirectorywatcherpreferencesfactory.hpp"
 #include "notemanager.hpp"
@@ -61,7 +62,7 @@ void NoteDirectoryWatcherApplicationAddin::initialize()
   m_signal_changed_cid = m_file_system_watcher->signal_changed()
     .connect(sigc::mem_fun(*this, &NoteDirectoryWatcherApplicationAddin::handle_file_system_change_event));
 
-  Glib::RefPtr<Gio::Settings> settings = gnote::Preferences::obj().get_schema_settings(SCHEMA_NOTE_DIRECTORY_WATCHER);
+  Glib::RefPtr<Gio::Settings> settings = ignote().preferences().get_schema_settings(SCHEMA_NOTE_DIRECTORY_WATCHER);
   m_check_interval = settings->get_int(CHECK_INTERVAL);
   sanitize_check_interval(settings);
   m_signal_settings_changed_cid = settings->signal_changed()
@@ -86,7 +87,7 @@ bool NoteDirectoryWatcherApplicationAddin::initialized()
 
 void NoteDirectoryWatcherApplicationAddin::handle_note_saved(const gnote::NoteBase::Ptr & note)
 {
-  m_note_save_times[note->id()] = sharp::DateTime::now();
+  m_note_save_times[note->id()] = Glib::DateTime::create_now_local();
 }
 
 void NoteDirectoryWatcherApplicationAddin::handle_file_system_change_event(
@@ -134,7 +135,7 @@ void NoteDirectoryWatcherApplicationAddin::handle_file_system_change_event(
       }
     }
 
-    record->second.last_change = sharp::DateTime::now();
+    record->second.last_change = Glib::DateTime::create_now_local();
   }
   catch(...)
   {}
@@ -166,15 +167,15 @@ bool NoteDirectoryWatcherApplicationAddin::handle_timeout()
 
       // Check that Note.Saved event didn't occur within (check-interval -2) seconds of last write
       if(m_note_save_times.find(iter.first) != m_note_save_times.end() &&
-          std::abs((m_note_save_times[iter.first] - iter.second.last_change).total_seconds()) <= (m_check_interval - 2)) {
+          std::abs(sharp::time_span_total_seconds(m_note_save_times[iter.first].difference(iter.second.last_change))) <= (m_check_interval - 2)) {
         DBG_OUT("NoteDirectoryWatcher: Ignoring (timeout) because it was probably a Gnote write");
         keysToRemove.push_back(iter.first);
         continue;
       }
       // TODO: Take some actions to clear note_save_times? Not a large structure...
 
-      sharp::DateTime last_change(iter.second.last_change);
-      if(sharp::DateTime::now() > last_change.add_seconds(4)) {
+      Glib::DateTime last_change(iter.second.last_change);
+      if(Glib::DateTime::create_now_local() > last_change.add_seconds(4)) {
         if(iter.second.deleted) {
           delete_note(iter.first);
         }
@@ -295,7 +296,7 @@ Glib::ustring NoteDirectoryWatcherApplicationAddin::make_uri(const Glib::ustring
 void NoteDirectoryWatcherApplicationAddin::on_settings_changed(const Glib::ustring & key)
 {
   if(key == CHECK_INTERVAL) {
-    Glib::RefPtr<Gio::Settings> settings = gnote::Preferences::obj().get_schema_settings(SCHEMA_NOTE_DIRECTORY_WATCHER);
+    Glib::RefPtr<Gio::Settings> settings = ignote().preferences().get_schema_settings(SCHEMA_NOTE_DIRECTORY_WATCHER);
     m_check_interval = settings->get_int(key);
     sanitize_check_interval(settings);
   }
