@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2017 Aurimas Cernius
+ * Copyright (C) 2012-2014,2017,2019-2020 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,31 +41,25 @@
 namespace gnote {
 namespace sync {
 
-  SyncManager::SyncManager(NoteManagerBase & m)
-    : m_note_manager(m)
+  SyncManager::SyncManager(IGnote & g, NoteManagerBase & m)
+    : m_gnote(g)
+    , m_note_manager(m)
     , m_state(IDLE)
     , m_sync_thread(NULL)
   {
   }
 
 
-  void SyncManager::init(NoteManagerBase & m)
+  void SyncManager::init()
   {
-    SyncManager *manager = new SyncManager(m);
-    manager->_init(m);
-  }
-
-
-  void SyncManager::_init(NoteManagerBase & manager)
-  {
-    m_client = GnoteSyncClient::create(manager);
+    m_client.reset(GnoteSyncClient::create(m_note_manager));
     // Add a "Synchronize Notes" to Gnote's Application Menu
-    IActionManager & am(IActionManager::obj());
+    IActionManager & am(m_gnote.action_manager());
     am.add_app_action("sync-notes");
-    am.add_app_menu_item(IActionManager::APP_ACTION_MANAGE, 200, _("Synchronize Notes"), "app.sync-notes");
+    am.add_app_menu_item(APP_SECTION_MANAGE, 200, _("Synchronize Notes"), "app.sync-notes");
 
     // Initialize all the SyncServiceAddins
-    initialize_sync_service_addins(manager);
+    initialize_sync_service_addins(m_note_manager);
 
     connect_system_signals();
 
@@ -90,7 +84,7 @@ namespace sync {
   {
     try {
       NoteManager & manager(dynamic_cast<NoteManager&>(note_mgr()));
-      Preferences::obj().get_schema_settings(Preferences::SCHEMA_SYNC)->signal_changed()
+      m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC)->signal_changed()
         .connect(sigc::mem_fun(*this, &SyncManager::preferences_setting_changed));
       manager.signal_note_saved.connect(sigc::mem_fun(*this, &SyncManager::handle_note_saved_or_deleted));
       manager.signal_note_deleted.connect(sigc::mem_fun(*this, &SyncManager::handle_note_saved_or_deleted));
@@ -133,11 +127,12 @@ namespace sync {
   void SyncManager::synchronization_thread()
   {
     struct finally {
+      SyncManager & manager;
       SyncServiceAddin *addin;
-      finally() : addin(NULL){}
+      finally(SyncManager & m) : manager(m), addin(NULL){}
       ~finally()
       {
-        SyncManager::_obj().m_sync_thread = NULL;
+        manager.m_sync_thread = NULL;
         try {
           if(addin) {
             addin->post_sync_cleanup();
@@ -147,8 +142,8 @@ namespace sync {
           ERR_OUT(_("Error cleaning up addin after synchronization: %s"), e.what());
         }
       }
-    } f;
-    SyncServer::Ptr server;
+    } f(*this);
+    std::unique_ptr<SyncServer> server;
     try {
       f.addin = get_configured_sync_service();
       if(f.addin == NULL) {
@@ -163,9 +158,9 @@ namespace sync {
 
       set_state(CONNECTING);
       try {
-        server = f.addin->create_sync_server();
+        server.reset(f.addin->create_sync_server());
         if(server == NULL)
-          throw new std::logic_error("addin.CreateSyncServer () returned null");
+          throw std::logic_error("addin.CreateSyncServer () returned null");
       }
       catch(std::exception & e) {
         set_state(SYNC_SERVER_CREATION_FAILED);
@@ -213,7 +208,7 @@ namespace sync {
 
       // Gather list of new/updated note titles
       // for title conflict handling purposes.
-      std::list<Glib::ustring> noteUpdateTitles;
+      std::vector<Glib::ustring> noteUpdateTitles;
       for(auto & iter : noteUpdates) {
         if(iter.second.m_title != "") {
           noteUpdateTitles.push_back(iter.second.m_title);
@@ -228,10 +223,10 @@ namespace sync {
       for(auto & iter : noteUpdates) {
         if(!find_note_by_uuid(iter.second.m_uuid) != 0) {
           NoteBase::Ptr existingNote = note_mgr().find(iter.second.m_title);
-          if(existingNote != 0 && !iter.second.basically_equal_to(static_pointer_cast<Note>(existingNote))) {
+          if(existingNote != 0 && !iter.second.basically_equal_to(std::static_pointer_cast<Note>(existingNote))) {
             DBG_OUT("Sync: Early conflict detection for '%s'", iter.second.m_title.c_str());
             if(m_sync_ui != 0) {
-              m_sync_ui->note_conflict_detected(static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
+              m_sync_ui->note_conflict_detected(std::static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
             }
           }
         }
@@ -255,21 +250,21 @@ namespace sync {
           existingNote = note_mgr().find(iter.second.m_title);
           if(existingNote != 0) {
             DBG_OUT("SyncManager: Deleting auto-generated note: %s", iter.second.m_title.c_str());
-            delete_note_in_main_thread(static_pointer_cast<Note>(existingNote));
+            delete_note_in_main_thread(std::static_pointer_cast<Note>(existingNote));
           }
           create_note_in_main_thread(iter.second);
         }
         else if(existingNote->metadata_change_date() <= m_client->last_sync_date()
-                || iter.second.basically_equal_to(static_pointer_cast<Note>(existingNote))) {
+                || iter.second.basically_equal_to(std::static_pointer_cast<Note>(existingNote))) {
           // Existing note hasn't been modified since last sync; simply update it from server
-          update_note_in_main_thread(static_pointer_cast<Note>(existingNote), iter.second);
+          update_note_in_main_thread(std::static_pointer_cast<Note>(existingNote), iter.second);
         }
         else {
           // Logger.Debug ("Sync: Late conflict detection for '{0}'", noteUpdate.Title);
           DBG_OUT("SyncManager: Content conflict in note update for note '%s'", iter.second.m_title.c_str());
           // Note already exists locally, but has been modified since last sync; prompt user
           if(m_sync_ui != 0) {
-            m_sync_ui->note_conflict_detected(static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
+            m_sync_ui->note_conflict_detected(std::static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
           }
 
           // Note has been deleted or okay'd for overwrite
@@ -277,7 +272,7 @@ namespace sync {
           if(existingNote == 0)
             create_note_in_main_thread(iter.second);
           else
-            update_note_in_main_thread(static_pointer_cast<Note>(existingNote), iter.second);
+            update_note_in_main_thread(std::static_pointer_cast<Note>(existingNote), iter.second);
         }
       }
 
@@ -285,16 +280,16 @@ namespace sync {
       // delegate to run in the main gtk thread.
       // To be consistent, any exceptions in the delgate will be caught
       // and then rethrown in the synchronization thread.
-      delete_notes_in_main_thread(server);
+      delete_notes_in_main_thread(*server);
 
       // TODO: Add following updates to syncDialog treeview
 
       set_state(PREPARE_UPLOAD);
       // Look through all the notes modified on the client
       // and upload new or modified ones to the server
-      std::list<Note::Ptr> newOrModifiedNotes;
-      FOREACH(const NoteBase::Ptr & iter, note_mgr().get_notes()) {
-        Note::Ptr note = static_pointer_cast<Note>(iter);
+      std::vector<Note::Ptr> newOrModifiedNotes;
+      for(const NoteBase::Ptr & iter : note_mgr().get_notes()) {
+        Note::Ptr note = std::static_pointer_cast<Note>(iter);
         if(m_client->get_revision(note) == -1) {
           // This is a new note that has never been synchronized to the server
           // TODO: *OR* this is a note that we lost revision info for!!!
@@ -321,8 +316,8 @@ namespace sync {
       }
 
       // Handle notes deleted on client
-      std::list<Glib::ustring> locallyDeletedUUIDs;
-      std::list<Glib::ustring> all_note_uuids = server->get_all_note_uuids();
+      std::vector<Glib::ustring> locallyDeletedUUIDs;
+      auto all_note_uuids = server->get_all_note_uuids();
       for(auto & iter : all_note_uuids) {
         if(find_note_by_uuid(iter) == 0) {
           locallyDeletedUUIDs.push_back(iter);
@@ -346,9 +341,8 @@ namespace sync {
       if(commitResult) {
         // Apply this revision number to all new/modified notes since last sync
         // TODO: Is this the best place to do this (after successful server commit)
-        for(std::list<Note::Ptr>::iterator iter = newOrModifiedNotes.begin();
-            iter != newOrModifiedNotes.end(); ++iter) {
-          m_client->set_revision(*iter, newRevision);
+        for(auto & iter : newOrModifiedNotes) {
+          m_client->set_revision(iter, newRevision);
         }
         set_state(SUCCEEDED);
       }
@@ -360,7 +354,7 @@ namespace sync {
       // This should be equivalent to newRevision
       m_client->last_synchronized_revision(server->latest_revision());
 
-      m_client->last_sync_date(sharp::DateTime::now());
+      m_client->last_sync_date(Glib::DateTime::create_now_local());
 
       DBG_OUT("Sync: New revision: %d", m_client->last_synchronized_revision());
 
@@ -401,8 +395,8 @@ namespace sync {
     // timer to avoid interupting the user (we want to
     // make sure not to sync more often than the user's pref)
     if(m_sync_thread == NULL) {
-      sharp::TimeSpan time_since_last_check = sharp::DateTime::now() - m_last_background_check;
-      if(time_since_last_check.total_minutes() > m_autosync_timeout_pref_minutes - 1) {
+      Glib::TimeSpan time_since_last_check = Glib::DateTime::create_now_local().difference(m_last_background_check);
+      if(sharp::time_span_total_minutes(time_since_last_check) > m_autosync_timeout_pref_minutes - 1) {
         DBG_OUT("Note edited...killing autosync timer until next save or delete event");
         m_autosync_timer.cancel();
       }
@@ -419,9 +413,9 @@ namespace sync {
 
   void SyncManager::update_sync_action()
   {
-    Glib::RefPtr<Gio::Settings> settings = Preferences::obj().get_schema_settings(Preferences::SCHEMA_SYNC);
+    Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC);
     Glib::ustring sync_addin_id = settings->get_string(Preferences::SYNC_SELECTED_SERVICE_ADDIN);
-    IActionManager::obj().get_app_action("sync-notes")->set_enabled(sync_addin_id != "");
+    m_gnote.action_manager().get_app_action("sync-notes")->set_enabled(sync_addin_id != "");
 
     int timeoutPref = settings->get_int(Preferences::SYNC_AUTOSYNC_TIMEOUT);
     if(timeoutPref != m_autosync_timeout_pref_minutes) {
@@ -430,7 +424,7 @@ namespace sync {
       if(m_autosync_timeout_pref_minutes > 0) {
         DBG_OUT("Autosync pref changed...restarting sync timer");
         m_autosync_timeout_pref_minutes = m_autosync_timeout_pref_minutes >= 5 ? m_autosync_timeout_pref_minutes : 5;
-        m_last_background_check = sharp::DateTime::now();
+        m_last_background_check = Glib::DateTime::create_now_local();
         // Perform a sync no sooner than user specified
         m_current_autosync_timeout_minutes = m_autosync_timeout_pref_minutes;
         m_autosync_timer.reset(m_current_autosync_timeout_minutes * 60000);
@@ -442,10 +436,9 @@ namespace sync {
   void SyncManager::handle_note_saved_or_deleted(const NoteBase::Ptr &)
   {
     if(m_sync_thread == NULL && m_autosync_timeout_pref_minutes > 0) {
-      sharp::TimeSpan time_since_last_check(sharp::DateTime::now() - m_last_background_check);
-      sharp::TimeSpan time_until_next_check(
-        sharp::TimeSpan(0, m_current_autosync_timeout_minutes, 0) - time_since_last_check);
-      if(time_until_next_check.total_minutes() < 1) {
+      Glib::TimeSpan time_since_last_check(Glib::DateTime::create_now_local().difference(m_last_background_check));
+      Glib::TimeSpan time_until_next_check = sharp::time_span(0, m_current_autosync_timeout_minutes, 0) - time_since_last_check;
+      if(sharp::time_span_total_minutes(time_until_next_check) < 1) {
         DBG_OUT("Note saved or deleted within a minute of next autosync...resetting sync timer");
         m_current_autosync_timeout_minutes = 1;
         m_autosync_timer.reset(m_current_autosync_timeout_minutes * 60000);
@@ -453,7 +446,7 @@ namespace sync {
     }
     else if(m_sync_thread == NULL && m_autosync_timeout_pref_minutes > 0) {
       DBG_OUT("Note saved or deleted...restarting sync timer");
-      m_last_background_check = sharp::DateTime::now();
+      m_last_background_check = Glib::DateTime::create_now_local();
       // Perform a sync one minute after setting change
       m_current_autosync_timeout_minutes = 1;
       m_autosync_timer.reset(m_current_autosync_timeout_minutes * 60000);
@@ -463,7 +456,7 @@ namespace sync {
 
   void SyncManager::background_sync_checker()
   {
-    m_last_background_check = sharp::DateTime::now();
+    m_last_background_check = Glib::DateTime::create_now_local();
     m_current_autosync_timeout_minutes = m_autosync_timeout_pref_minutes;
     if(m_sync_thread != NULL) {
       return;
@@ -471,9 +464,9 @@ namespace sync {
     SyncServiceAddin *addin = get_configured_sync_service();
     if(addin) {
       // TODO: block sync while checking
-      SyncServer::Ptr server;
+      std::unique_ptr<SyncServer> server;
       try {
-        server = SyncServer::Ptr(addin->create_sync_server());
+        server.reset(addin->create_sync_server());
         if(server == 0) {
           throw std::logic_error("addin->create_sync_server() returned null");
         }
@@ -487,8 +480,8 @@ namespace sync {
       bool server_has_updates = false;
       bool client_has_updates = m_client->deleted_note_titles().size() > 0;
       if(!client_has_updates) {
-        FOREACH(const NoteBase::Ptr & iter, note_mgr().get_notes()) {
-          Note::Ptr note = static_pointer_cast<Note>(iter);
+        for(const NoteBase::Ptr & iter : note_mgr().get_notes()) {
+          Note::Ptr note = std::static_pointer_cast<Note>(iter);
           if(m_client->get_revision(note) == -1 || note->metadata_change_date() > m_client->last_sync_date()) {
             client_has_updates = true;
             break;
@@ -514,7 +507,7 @@ namespace sync {
       if(client_has_updates || server_has_updates) {
         DBG_OUT("Detected that sync would be a good idea now");
         // TODO: Check that it's safe to sync, block other sync UIs
-        perform_synchronization(SilentUI::create(note_mgr()));
+        perform_synchronization(SilentUI::create(m_gnote, note_mgr()));
       }
     }
 
@@ -540,7 +533,7 @@ namespace sync {
   {
     SyncServiceAddin *addin = NULL;
 
-    Glib::ustring sync_service_id = Preferences::obj()
+    Glib::ustring sync_service_id = m_gnote.preferences()
       .get_schema_settings(Preferences::SCHEMA_SYNC)->get_string(Preferences::SYNC_SELECTED_SERVICE_ADDIN);
     if(sync_service_id != "") {
       addin = get_sync_service_addin(sync_service_id);
@@ -556,11 +549,10 @@ namespace sync {
 
     try {
       NoteManager & manager(dynamic_cast<NoteManager&>(m_note_manager));
-      std::list<SyncServiceAddin*> addins;
-      manager.get_addin_manager().get_sync_service_addins(addins);
-      for(std::list<SyncServiceAddin*>::iterator iter = addins.begin(); iter != addins.end(); ++iter) {
-        if((*iter)->id() == sync_service_id) {
-          addin = *iter;
+      std::vector<SyncServiceAddin*> addins = manager.get_addin_manager().get_sync_service_addins();
+      for(auto iter : addins) {
+        if(iter->id() == sync_service_id) {
+          addin = iter;
           break;
         }
       }
@@ -611,7 +603,7 @@ namespace sync {
     }
     catch(...)
     {} // TODO: Handle exception in case that serverNote.XmlContent is invalid XML
-    m_client->set_revision(static_pointer_cast<Note>(localNote), serverNote.m_latest_revision);
+    m_client->set_revision(std::static_pointer_cast<Note>(localNote), serverNote.m_latest_revision);
 
     // Update dialog's sync status
     if(m_sync_ui != 0) {
@@ -678,25 +670,25 @@ namespace sync {
   }
 
 
-  void SyncManager::delete_notes_in_main_thread(const SyncServer::Ptr & server)
+  void SyncManager::delete_notes_in_main_thread(SyncServer & server)
   {
-    utils::main_context_call([this, server]() { delete_notes(server); });
+    utils::main_context_call([this, &server]() { delete_notes(server); });
   }
 
 
-  void SyncManager::delete_notes(const SyncServer::Ptr & server)
+  void SyncManager::delete_notes(SyncServer & server)
   {
     try {
       // Make list of all local notes
-      std::list<NoteBase::Ptr> localNotes = note_mgr().get_notes();
+      auto localNotes = note_mgr().get_notes();
 
       // Get all notes currently on server
-      auto serverNotes = server->get_all_note_uuids();
+      auto serverNotes = server.get_all_note_uuids();
 
       // Delete notes locally that have been deleted on the server
-      FOREACH(const NoteBase::Ptr & iter, localNotes) {
-        Note::Ptr note = static_pointer_cast<Note>(iter);
-	if(SyncManager::_obj().m_client->get_revision(note) != -1
+      for(const NoteBase::Ptr & iter : localNotes) {
+        Note::Ptr note = std::static_pointer_cast<Note>(iter);
+	if(m_client->get_revision(note) != -1
 	   && std::find(serverNotes.begin(), serverNotes.end(), note->id()) == serverNotes.end()) {
 	  if(m_sync_ui != 0) {
 	    m_sync_ui->note_synchronized(note->get_title(), DELETE_FROM_CLIENT);
