@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2017-2018 Aurimas Cernius
+ * Copyright (C) 2012-2014,2017-2020 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,11 +27,13 @@
 #include <glibmm/main.h>
 #include <glibmm/thread.h>
 
-#include "base/macros.hpp"
 #include "isyncmanager.hpp"
 
 
 namespace gnote {
+
+  class IGnote;
+
 namespace sync {
 
   class SyncServiceAddin;
@@ -40,8 +42,8 @@ namespace sync {
     : public ISyncManager
   {
   public:
-    SyncManager(NoteManagerBase &);
-    static void init(NoteManagerBase &);
+    SyncManager(IGnote &, NoteManagerBase &);
+    void init();
     virtual void reset_client() override;
     virtual void perform_synchronization(const SyncUI::Ptr & sync_ui) override;
     void synchronization_thread();
@@ -51,13 +53,17 @@ namespace sync {
       {
         return m_state;
       }
+    virtual SyncUtils & utils() override
+      {
+        return m_utils;
+      }
   protected:
     virtual void initialize_sync_service_addins(NoteManagerBase &);
     virtual void connect_system_signals();
     virtual SyncServiceAddin *get_sync_service_addin(const Glib::ustring & sync_service_id);
     virtual SyncServiceAddin *get_configured_sync_service();
-    virtual void delete_notes_in_main_thread(const SyncServer::Ptr & server);
-    void delete_notes(const SyncServer::Ptr & server);
+    virtual void delete_notes_in_main_thread(SyncServer & server);
+    void delete_notes(SyncServer & server);
     virtual void note_save(const Note::Ptr & note);
     virtual void create_note_in_main_thread(const NoteUpdate & noteUpdate);
     virtual void update_note_in_main_thread(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate);
@@ -66,14 +72,9 @@ namespace sync {
     void update_note(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate);
     void delete_note(const Note::Ptr & existingNote);
 
-    SyncClient::Ptr m_client;
+    std::unique_ptr<SyncClient> m_client;
     SyncUI::Ptr m_sync_ui;
   private:
-    static SyncManager & _obj()
-      {
-        return static_cast<SyncManager&>(obj());
-      }
-    void _init(NoteManagerBase &);
     void handle_note_saved_or_deleted(const NoteBase::Ptr & note);
     void handle_note_buffer_changed(const NoteBase::Ptr & note);
     void preferences_setting_changed(const Glib::ustring & key);
@@ -85,6 +86,8 @@ namespace sync {
     NoteManagerBase & note_mgr();
     void get_synchronized_xml_bits(const Glib::ustring & noteXml, Glib::ustring & title, Glib::ustring & tags, Glib::ustring & content);
 
+    SyncUtils m_utils;
+    IGnote & m_gnote;
     NoteManagerBase & m_note_manager;
     SyncState m_state;
     Glib::Thread *m_sync_thread;
@@ -92,7 +95,7 @@ namespace sync {
     utils::InterruptableTimeout m_autosync_timer;
     int m_autosync_timeout_pref_minutes;
     int m_current_autosync_timeout_minutes;
-    sharp::DateTime m_last_background_check;
+    Glib::DateTime m_last_background_check;
   };
 
 
